/* Olympic High Jump v0.021 by Shrodes
 * 
 * Special Thanks: Map/radar code by Salakavala (thanks!!!). Idea from High Jump by ?. Contain aspects from Allasturvat and   * Shroditurvat.
 * 
 * Version History
 *
 * v0.021-Added (modified) map/radar from Salakavala's Target Dismount. Added revert colour function. Fixed level layout      * (new bar, longer pole, more bounce off at the start.) YES! Fixed dead text glitch AND foot being stuck in launcher on 1st  * shot glitch!! New rank!
 *
 * v0.02- Added ranks, deleted number 50 at 50m mark (just look at the top corner!!!), added top corner height show back      * in! Changed ranks and their score reqs.
 *
 * v0.01b-Initial level making and coding-you know the drill! 
 * 
 */

/* Note: playSound function is undocumented as it is currently
 *       only temporarily as it is.
 */

const int BONUS_OBJ_SCORE = 1500;
const int ROTO_SCORE = 500;
const int ELEVATOR_SCORE = 1000;
const int MAX_TIME = 0;

uint mTime;
int screenWidth, screenHeight;
uint frame = 0;

int timeLeft, timeBonus = 0;
int score = 0;
int somersaults = 0;
int finalsomersaults = -1;

float height = 0;
float xPos = 0, yPos = 0, zPos = 0;

GameApp *app;
UIFont *medFont, *smallFont;
uint medFontHeight, smallFontHeight;
PhysicsWorld *pw;

PhysicsObject *head, *neck, *trunk, *rthigh, *lthigh, *rcalf, *lcalf, *rfore, *lfore, *larm, *rarm, *rfoot, *lfoot;

PhysicsObject *objHead;

PhysicsObject *objH1, *objH2, *objH3, *objH4, *objH5, *objH6;
uint objHColor;
PhysicsObject *collectedBonus = 0;

PhysicsObject *objRoto, *objButton;
float rotoInitAng, rotoAng = 0;
uint objButtonBlockColor;

PhysicsObject *objElevator, *objElevTrig;
int elevAct = 0, elevTimer = 0;
float elevPosX, elevPosY, elevPosZ;
float elevColR, elevColG, elevColB;

PhysicsObject *objFinish;
PhysicsObject *bar;

bool finished = false;
bool aborted = false;

bool firstTouch = false;
bool secondTouch = false;

bool headAbove = true;

bstr first = "";
bstr second = "";


// Called once on level initialization.
void onInit()
{
    app = getGameApp();
    screenWidth = app->getScreenWidth();
    screenHeight = app->getScreenHeight();
    medFont = app->getFont("medium");
    smallFont = app->getFont("small");
    medFontHeight = medFont->getFontHeight();
    smallFontHeight = smallFont->getFontHeight();
    
    app->setClearColor(0.2, 0.2, 0.8);
    
    pw = app->getPhysicsWorld();
    pw->setGravity(0, 0, -0.5);

    objHead = pw->findObject("Head");

	head = pw->findObject("Head");
	neck = pw->findObject("Neck");
	trunk = pw->findObject("Trunk");
	rthigh = pw->findObject("RightThigh");
	lthigh = pw->findObject("LeftThigh");
	rarm = pw->findObject("RightArm");
	larm = pw->findObject("LeftArm");
	rfore = pw->findObject("RightForearm");
	lfore = pw->findObject("LeftForearm");
	rcalf = pw->findObject("RightCalf");
	lcalf = pw->findObject("LeftCalf");
	rfoot = pw->findObject("RightFoot");
	lfoot = pw->findObject("LeftFoot");
	
	revertColors();

    objH1 = pw->findObject("H1");
    objH2 = pw->findObject("H2");
    objH3 = pw->findObject("H3");
    objH4 = pw->findObject("H4");
    objH5 = pw->findObject("H5");
    objH6 = pw->findObject("H6");
    objHColor = objH1->getBlockColor();

    float y, p, r;
    objRoto = pw->findObject("Roto");
    objRoto->getInitialRotation(y, p, r);
    rotoInitAng = r;

    objButton = pw->findObject("Button");
    objButtonBlockColor = objButton->getBlockColor();

    objElevator = pw->findObject("Elevator");
    objElevator->getInitialPosition(elevPosX, elevPosY, elevPosZ);
    objElevator->getInitialColor(elevColR, elevColG, elevColB);

    objElevTrig = pw->findObject("ElevTrig");

    objFinish = pw->findObject("objFinish");
    bar = pw->findObject("bar");
}

void revertColors() {
	head->setColor(1,0.75,0.75);
	neck->setColor(1,0.75,0.75);
	trunk->setColor(0.75,0.75,0.75);
	rthigh->setColor(0,0,1);
	lthigh->setColor(0,0,1);
	rarm->setColor(0.75,0.75,0.75);
	larm->setColor(0.75,0.75,0.75);
	rfore->setColor(1,0.75,0.75);
	lfore->setColor(1,0.75,0.75);
	rcalf->setColor(1,0.75,0.75);
	lcalf->setColor(1,0.75,0.75);
	rfoot->setColor(0,0,0);
	lfoot->setColor(0,0,0);
}


// Called once for each game logic update before the physics update.
// Parameter 'time' is the simulation time in milliseconds.
void onUpdate(uint time)
{
    ++frame;
    mTime = time;
    
    float x, y, z = 0;
	head->getPosition(x, y, z);
	float headZ = z;
	if (!aborted && z < -50 && !finished)
	{
		aborted = true;
		setGameState(GAMESTATE_ABORTED);
	}
    
    trunk->getPosition(x,y,z);
	float trunkZ = z;
	if (z > height) height = z;
	
	if (!headAbove && headZ > trunkZ) {
		headAbove = true;
		somersaults++;
	} else if (headAbove && headZ < trunkZ) {
		headAbove = false;
	}
{
	trunk->getPosition(xPos, yPos, zPos);
}

    if (rotoAng > 0 && (frame % 2) == 0)
    {
        // rotate object only every other frame so that
        // there's a bit more time for the collisions to resolve
        objRoto->setRotation(0, 0, rotoInitAng - rotoAng);
        rotoAng -= PI / 60;
    }

    if (elevAct != 0)
    {
        if (elevAct > 0)
        {
            elevPosZ += 0.06f;
            --elevAct;
        }
        else
        {
            elevPosZ -= 0.06f;
            ++elevAct;
        }
        if ((frame % 2) == 0)
            objElevator->setPosition(elevPosX, elevPosY, elevPosZ);
        objElevator->setColor(elevColR, elevColG, elevColB * 1.5f);
    }
    else
        objElevator->setColor(elevColR, elevColG, elevColB);
    if (elevTimer > 0)
    {
        --elevTimer;
        if (elevTimer == 0)
        {
            elevAct = -100;
            playSound(5); // elevator sound
        }
    }

    if (collectedBonus != 0)
    {
        collectedBonus->removeFromPhysicsWorld();
        collectedBonus = 0;
    }
}


// Called when there is a collision between the two given objects.
// Returns true to create contacts or false to ignore collision.
bool onCollision(PhysicsObject *o1, PhysicsObject *o2) {
		bstr o1name = o1->getName();
		bstr o2name = o2->getName();
		
		if (o1->getType() == PHYSOBJ_RAGDOLL_BODY) o1->setColor(1,0,0);
		if (o2->getType() == PHYSOBJ_RAGDOLL_BODY) o2->setColor(1,0,0);

    uint o1bc = o1->getBlockColor(), o2bc = o2->getBlockColor();
    
    if ((o1->getType() == PHYSOBJ_RAGDOLL_BODY && o2name == "bouncer") || (o2->getType() == PHYSOBJ_RAGDOLL_BODY && o1name == "bouncer")) 
    {
			somersaults = 0;
			height = 0;
    }

    PhysicsObject *bonusObj = 0;
    if (o2bc == objHColor &&
        (o1 == objH1 || o1 == objH2 || o1 == objH3 ||
         o1 == objH4 || o1 == objH5 || o1 == objH6))
        bonusObj = o1;
    if (o1bc == objHColor &&
        (o2 == objH1 || o2 == objH2 || o2 == objH3 ||
         o2 == objH4 || o2 == objH5 || o2 == objH6))
        bonusObj = o2;
    if (bonusObj != 0)
    {
        score += BONUS_OBJ_SCORE;
        // object cannot be removed from physics world in onCollision callback
        collectedBonus = bonusObj;
        playSound(3); // bonus sound
    }

    // Trigger roto object if the same colored hand hits the button.
    if (((o1 == objButton && o2bc == objButtonBlockColor) ||
         (o2 == objButton && o1bc == objButtonBlockColor)) && rotoAng <= 0)
    {
        score += ROTO_SCORE;
        rotoAng = PI;
        playSound(4); // roto sound
    }

    // Check if some object hits the elevator trigger (no collisions).
    if ((o1 == objElevTrig || o2 == objElevTrig) && elevAct == 0 && elevTimer == 0)
    {
        // Move elevator after ragdoll hits the elevator trigger.
        if (o1->getType() == PHYSOBJ_RAGDOLL_BODY ||
            o2->getType() == PHYSOBJ_RAGDOLL_BODY)
        {
            elevAct = 100;
            elevTimer = 480;
            score += ELEVATOR_SCORE;
            playSound(5); // elevator sound
        }
        // no collision
        return false;
    }
    if ((o1name == "objFinish" && o2name != "objFinish") || (o2name == "objFinish" && o1name != "objFinish")  && !finished && !firstTouch) {
		if (o1name != "objFinish") first = o1name;
		else first = o2name;
		firstTouch = true;
		return true;
    }
    
    if (!aborted  && ((o2name == "bar" && o1->getType() == PHYSOBJ_RAGDOLL_BODY) || (o1name == "bar" && o2->getType() == PHYSOBJ_RAGDOLL_BODY))&& !finished) 
    	{
		aborted = true;
		setGameState(GAMESTATE_ABORTED);
	}
	if ((o1name == "objFinish" && o2name != "objFinish") || (o2name == "objFinish" && o1name != "objFinish")  && !finished && firstTouch) {
	  
		if (o1name != "objFinish" && o1name != first) {
			second = o1name;
			secondTouch = true;
		} else if (o2name != "objFinish" && o2name != first) {
			second = o2name;
			secondTouch = true;
		}

		if (secondTouch) {
		        finished = true;
		// setGameState(GAMESTATE_FINISHED);
		
		}
	}
    // generate collision
    return true;
}


// Called when the game view is rendered.
// Might happen more or less frequently than update calls.
void onRender() 
{
	drawMap();
    
    if (mTime < 5000) {
		medFont->drawString(float(screenWidth / 2), 70, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_BOTTOM, "Olympic High Jump v0.021",1,1,0,1);
		smallFont->drawString(float(screenWidth / 2), 90, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_BOTTOM, "by Shrodes");
		smallFont->drawString(float(screenWidth / 2), 110, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_BOTTOM, "Jump as high as you can and land on the green finish platform- just don't hit the bar!");
    }
	if (!finished) {
	smallFont->drawString(100,20, UIFONT_ALIGN_HORIZ_CENTER, "Height: " + bstrFormat(height) + "m");
	}
	if (aborted)
		drawFailure();
	if (finished)
	drawVictory();
}
void drawMap() {

	int x = int(xPos);
	int y = int(yPos);
	int z = int(zPos/2);

	for (int i = 10; i<20; i+=5) {
		for (x = 0; x<40-i*2; x++) {
			drawPixel(160+i+x,100+i, 1-float(i)/30,1,0, 0.3);
			drawPixel(160+i+x,140-i, 1-float(i)/30,1,0, 0.3);
		}

		for (y = 0; y<40-i*2; y++) {
			drawPixel(160+i,100+i+y,1-float(i)/30,1,0, 0.3);
			drawPixel(200-i,100+i+y,1-float(i)/30,1,0, 0.3);
		}
	}

	drawPixel(80, 50 + (100 - z), 0,0,0,1);
	for (int y = 50; y<150; y++) {
		drawPixel(78, y, 1,1,1,0.3);
		drawPixel(82, y, 1,1,1,0.3);
	}

	drawPixel(130,130, 1, 0, 0, 0.3);
	drawPixel(130,131, 1, 0, 0, 0.3);
	drawPixel(131,130, 1, 0, 0, 0.3);
	drawPixel(131,131, 1, 0, 0, 0.3);

	drawPixel(int(150-xPos*1),int(120+yPos*2), 0, 0, 0, 1);

}

void drawPixel(int x, int y, float r, float g, float b, float a) {
	smallFont->drawString(float(x), float(y), UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, ".", r, g, b, a);
}



void drawHeight()
{
    smallFont->drawString(float(screenWidth / 3), float(screenHeight / 3), UIFONT_ALIGN_HORIZ_CENTER, "Height: " + bstrFormat(height) + "m");
}

    void drawFailure()
{
	medFont->drawString(float(screenWidth / 2), float(screenHeight / 2),
                        UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER,
                        "WHAT you died!? Its not that hard, just jump and land!!!");
}

void drawVictory()
{
	float r=0, g=0, b=0, a=1;

	medFont->drawString(float(screenWidth / 2), float(screenHeight / 4 + int(medFontHeight) * 2), UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "The judges have tallied your score and:",1,1,0,1);  
	medFont->drawString(float(screenWidth / 2), float(screenHeight / 4 + int(medFontHeight) * 5), UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Your height was: " + bstrFormat(height) + "m",0,0,0,1);
        medFont->drawString(float(screenWidth / 2), float(screenHeight / 4 + int(medFontHeight) * 7), UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, " ___________________",1,0,0,1);

	int x=6;

	medFont->drawString(float(screenWidth / 2), float(screenHeight / 3 + int(medFontHeight) * 6), UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "|Your final score was: " + bstrFormat(height*1000),1,0,0,1);

	medFont->drawString(float(screenWidth / 2), float(screenHeight / 3 + int(medFontHeight) * 6), UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "                                |",1,0,0,1);

	medFont->drawString(float(screenWidth / 2), float(screenHeight / 3 + int(medFontHeight) * 6), UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "  ___________________",1,0,0,1);

	bstr title = "Your country is now banned from any further use of you! (10th)";

	if (height*1000 > 1000) {
		title = "\"It is a MIRACLE that you finished!\" (9th)";
		r = 1;
		g = 0;
		b = 0;
	}
	if (height*1000 > 2500) {
		title = "\"That was terrible\" (8th)";
		r = 1;
		g = 0.33;
		b = 0;
	}
	if (height*1000 > 4000) {
		title = "\"How did you qualify for the Olympics?\" (7th)";
		r = 1;
		g = 0;
		b = 1;
	}

	if (height*1000 > 55000) {
		title = "\"No award for you, my friend.\" (6th)";
		r = 1;
		g = 0;
		b = 1;
	}
	if (height*1000 > 70000) {
		title = "\"And the grasshopper trophy goes to...\" (5th)";
		r = 1;
		g = 1;
		b = 0;
	}
	if (height*1000 > 85000) {
		title = "\"And the try-hard trophy goes to...\" (4th)";
		r = 0;
		g = 0;
		b = 1;
	}
	if (height*1000 > 100000) {
		title = "\"And the bronze medal goes to...\" (3rd)";
		r = 0;
		g = 1;
		b = 0;
	} 
	if (height*1000 > 115000) {
		title = "\"And the silver medal goes to...\" (2nd)";
		r = 1;
		g = 0;
		b = 1;
	}
	if (height*1000 > 130000) {
		title = "\"And the gold medal goes to...\" (1st)";
		r = 0;
		g = 0;
		b = 0;
	}
	if (height*1000 > 400000) {
		title = "\"The Salakavala medal goes to...\" (There Salakavala!)";
		r = 0;
		g = 0;
		b = 0;
	}

	medFont->drawString(float(screenWidth / 2), float(screenHeight / 4 + int(medFontHeight) * (x+6)), UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "The Olympic Judges say:");
	medFont->drawString(float(screenWidth / 2), float(screenHeight / 4 + int(medFontHeight) * (x+7)), UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, title, r, g, b, a);
	}